package cn.com.qmth.scancentral.service.impl;

import cn.com.qmth.scancentral.dao.ScannerDao;
import cn.com.qmth.scancentral.entity.AnswerCardEntity;
import cn.com.qmth.scancentral.entity.ScannerEntity;
import cn.com.qmth.scancentral.entity.SystemConfigEntity;
import cn.com.qmth.scancentral.service.AnswerCardService;
import cn.com.qmth.scancentral.service.AnswerCardSubjectService;
import cn.com.qmth.scancentral.service.ScannerService;
import cn.com.qmth.scancentral.service.SessionService;
import cn.com.qmth.scancentral.vo.scannerinfo.Control;
import cn.com.qmth.scancentral.vo.scannerinfo.ScannerInfo;
import cn.com.qmth.scancentral.vo.scannerinfo.ScannerInfoCard;
import cn.com.qmth.scancentral.vo.scannerinfo.ScannerInfoVo;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.core.exception.ParameterException;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service
public class ScannerServiceImpl extends ServiceImpl<ScannerDao, ScannerEntity> implements ScannerService {

    @Autowired
    private SystemConfigServiceImpl systemConfigService;

    @Autowired
    private SessionService sessionService;

    @Autowired
    private AnswerCardService answerCardService;

    @Autowired
    private AnswerCardSubjectService answerCardSubjectService;

    @Transactional
    @Override
    public void onLogin(String device) {
        if (StringUtils.isBlank(device)) {
            throw new ParameterException("设备信息不能为空");
        }
        ScannerEntity e = new ScannerEntity();
        e.setDevice(device);
        e.setLastLoginTime(new Date().getTime());
        this.saveOrUpdate(e);
    }

    @Override
    public ScannerInfoVo scannerInfo(Long examId) {
        ScannerInfoVo vo = new ScannerInfoVo();
        vo.setControl(new Control());
        SystemConfigEntity config = systemConfigService.find();
        if (config == null) {
            vo.getControl().setEnableLogin(true);
        } else {
            vo.getControl()
                    .setEnableLogin(config.getScannerEnableLogin() == null ? true : config.getScannerEnableLogin());
            vo.getControl().setPassword(config.getScannerPassword());
        }
        List<ScannerInfo> infos = baseMapper.getInfoByExam(examId);
        if (infos == null) {
            infos = new ArrayList<>();
        }
        vo.setList(infos);
        for (ScannerInfo s : infos) {
            if (s.getAnswerCardNumber() != null) {
                AnswerCardEntity card = answerCardService.findByExamAndNumber(examId, s.getAnswerCardNumber());
                ScannerInfoCard c = new ScannerInfoCard();
                c.setNumber(card.getNumber());
                c.setCode(card.getCode());
                c.setRemark(card.getRemark());
                c.setSubjectList(answerCardSubjectService.findByExamAndNumber(examId, card.getNumber()));
                s.setAnswerCard(c);
            }
            s.setOnline(sessionService.checkScannerOnline(s.getDevice()));
        }
        return vo;
    }

}
